
/* CZ-101 librarian is a program for uploading
and downloading sound banks from Casio CZ-101
synthesizers. It carefully handles .syx (sysex)
System exclusive files that often have erroneous
information within them. It is designed to avoid
midi errors by using flow control with CZ-101 */


#include <windows.h>
#include <wingdi.h>
#include <commdlg.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <shellapi.h>
#include <direct.h>
#include <mmsystem.h>
#include <time.h>
#include <winbase.h>

/* Global Variables Section */

/* Midi Storage */
MIDIHDR minbuf_hdr;
MIDIHDR moutbuf_hdr;
HMIDIOUT out_handle;
HMIDIIN in_handle;
char midi_in_buffer[1000];
char midi_out_buffer[1000];
int midi_channel_num=1;

/* Midi device 0 = none, so (+1) */
int midi_device_in=0;
int midi_device_out=0;
int midi_in_devices;
int midi_out_devices;

FILE *syxfile;

int status_code=0;
int error_code=0;

/* cz_memory 0=preset 1=internal 2=cartridge */
int cz_memory=1;
int patches_uploaded;
int patches_downloaded;
int text_value;

/* For sending single patch mode */
int patch_disc=0;
int patch_cz=0;
int patch_single_send;

/* Windows globals */

HINSTANCE application_inst;
HWND cz_window;
HBITMAP cz_about_bmp;
char base_startup[500];

char cz_prog_name[]="CZ-101 Librarian";

/* Prototypes Section */

LRESULT CALLBACK cz_proc_a (HWND,UINT,WPARAM,LPARAM);
LRESULT CALLBACK cz_proc_b (HWND,UINT,WPARAM,LPARAM);
LRESULT CALLBACK cz_proc_c (HWND,UINT,WPARAM,LPARAM);
LRESULT CALLBACK cz_proc_d (HWND,UINT,WPARAM,LPARAM);

int send_bank(void);
int receive_bank(void);
int push_out_patch(int);
int midi_startin(void);
int midi_send(int);
int midi_receive(void);
int midi_pause(int);
int midi_init_dialog(HWND);
int patch_init_dialog(HWND);
int cz_test_online(void);
int draw_cz_window(void);

/* Include Program Subroutines Section */

#include "resource.h"
#include "czsend.cpp"
#include "czrec.cpp"
#include "czwproc.cpp"
#include "czmidi.cpp"

/* The Winmain Main Program */

int PASCAL WinMain (HINSTANCE hInst,HINSTANCE hPreInst,
 LPSTR lpszCmdLine,int nCmdShow)

{
 MSG       lpMsg;
 WNDCLASS  cz_app;
 int       count;
 
 strcpy(base_startup,_pgmptr);
 count=strlen(base_startup)-1;
 while(base_startup[count]!='\\')
 {
  count--; 
 }
 base_startup[count+1]='\0';
 
 application_inst=hInst;
 cz_about_bmp=LoadBitmap(hInst,"CZ_ABOUT_BMP");
 
 midi_in_devices=midiInGetNumDevs(); 
 midi_out_devices=midiOutGetNumDevs();

 if (!hPreInst) 
 {
  cz_app.lpszClassName=cz_prog_name;
  cz_app.hInstance=hInst;
  cz_app.lpfnWndProc=cz_proc_a;
  cz_app.hCursor=LoadCursor(NULL,IDC_CROSS);
  cz_app.hIcon=LoadIcon(hInst,"CZ_ICON");
  cz_app.lpszMenuName="CZ_MENU";
  cz_app.hbrBackground=(HBRUSH)GetStockObject(BLACK_BRUSH);
  cz_app.style=CS_BYTEALIGNWINDOW;
  cz_app.cbClsExtra=0;
  cz_app.cbWndExtra=0;

  /* Exit application if can not create main window */
  if (!RegisterClass (&cz_app))
    return(FALSE);
 }

 cz_window=CreateWindow(cz_prog_name,"CZ-101 Librarian",
  WS_CAPTION|WS_SYSMENU| 
  WS_MINIMIZEBOX|WS_CLIPCHILDREN,
  CW_USEDEFAULT,CW_USEDEFAULT,260+2+
  2*GetSystemMetrics(SM_CXBORDER)+   
  2*GetSystemMetrics(SM_CXEDGE),65+2+
  2*GetSystemMetrics(SM_CYBORDER)+
  2*GetSystemMetrics(SM_CYEDGE)+
  GetSystemMetrics(SM_CYMENU)+
  GetSystemMetrics(SM_CYCAPTION),
  (HWND) NULL,
  (HMENU) NULL, (HINSTANCE) hInst,
  (LPSTR) NULL);
  ShowWindow(cz_window,nCmdShow);
  UpdateWindow(cz_window);

 if (cz_window==NULL) return(FALSE);

 while (GetMessage(&lpMsg,NULL,0,0)) 
 {
  TranslateMessage(&lpMsg);
  DispatchMessage(&lpMsg);
  }
 return (lpMsg.wParam);
}
