int send_bank(void)

/* send_bank subroutine accepts a disk file, and sends any
256 byte data segment preceeded with a CZ-101 sysex send 
patch command. It will send the same patch number again, or 
fail to send any if no 256 byte data areas, or CZ-101
send headers are present in the file */

{
 
 int input_char;
 int buffer_location=0;
 int patch_num;
 int send_status=0;
 int byte=0;
 int retries=0;
 int proceedable;

 patches_uploaded=0;
 while ((input_char=fgetc(syxfile))!=-1)
 {
   byte++;
   if (input_char==0xF0) 
   { 
	send_status=1;
	byte=1;
   }
   
   /* Expect the string 44 00 00 70+(nC) 20 (nP) hex 
   after F0 or go to next F0 - ie. Ignore data until next
   marker. This repeating code eliminates possible EOFs,
   as detected above

   (nP) denotes the midi channel
   
   (nC) denotes the patch number (1-16) and storage
   memory (high nibble), which gets trunced later */ 

   if ((byte==2)&&(input_char!=0x44)) send_status=0;
   if ((byte==3)&&(input_char!=0x00)) send_status=0;
   if ((byte==4)&&(input_char!=0x00)) send_status=0;
   
   /* (receive = 0x10)  
   We are expecting the data in sysex send command
   style (send = 0x20) */

   if ((byte==6)&&(input_char!=0x20)) send_status=0;

   if (byte==7) patch_num=(input_char&0x0F);
   if ((byte==8)&&(send_status==1)) send_status=2;
 
   /* The 256 byte absorption loop

   Note: Reading a F0 prior to reading 256 bytes
   causes the block to never be released as
   status=0 */
   
   if (send_status==2)
   {
    midi_out_buffer[buffer_location+7]=input_char;
	if (buffer_location==255) send_status=3;
	buffer_location++;
   } 
	
   if (send_status==3) 
   {
	buffer_location=0;
	retries=0;
    
    proceedable=0;
	if (patch_single_send==0) proceedable=1;
	if ((patch_single_send==1) && (patch_disc==patch_num))
	 proceedable=1;

	if (proceedable==1)
	{

	 /* The send retries */
	 while (retries<=5)
	 {
	  retries++;
	 
	  if (push_out_patch(patch_num)==1) retries=20;
	  if (retries==6) 
	  {
	   error_code=2;
       draw_cz_window();
	   return (patches_uploaded);
      }
     }
	}
    send_status=0;
   }
 }

 status_code=4;
 text_value=patches_uploaded;
 draw_cz_window();

 return (patches_uploaded);
}


int push_out_patch(int patch_num)
{
 int ack_ok;
 
 if (patch_single_send==1) patch_num=patch_cz;
 ack_ok=1;
 midi_out_buffer[0]=(char)0xF0;
 midi_out_buffer[1]=(char)0x44;
 midi_out_buffer[2]=(char)0x00;
 midi_out_buffer[3]=(char)0x00;
 midi_out_buffer[4]=(char)0x70+(midi_channel_num-1);
 midi_out_buffer[5]=(char)0x20;
 if (cz_memory==0) midi_out_buffer[6]=0x00+patch_num;
 if (cz_memory==1) midi_out_buffer[6]=0x20+patch_num;
 if (cz_memory==2) midi_out_buffer[6]=0x40+patch_num;
 
 midi_out_buffer[263]=(char)0xF7;

 midi_startin();

 /* Send patch sysex command, form is: 
 F0 44 00 00 [70+ch] 20 [slot] [data1]...[data256] F7 */
 
 midi_send(264);
 midi_pause(1300);
 if (midi_receive()<7) return(0);

 if (midi_in_buffer[0]!=(char)0xF0) ack_ok=0;
 if (midi_in_buffer[1]!=(char)0x44) ack_ok=0;
 if (midi_in_buffer[2]!=(char)0x00) ack_ok=0;
 if (midi_in_buffer[3]!=(char)0x00) ack_ok=0;
 if (midi_in_buffer[5]!=(char)0x30) ack_ok=0;
 if (midi_in_buffer[6]!=(char)0xF7) ack_ok=0;

 if (ack_ok==0) return(0);
 patches_uploaded++;

 status_code=3;
 text_value=(patch_num+1);
 draw_cz_window();

 /* 1 indicates patch sent properly*/
 return (1);
} 


